/////////////////////////////////////////////////////////////////////////////
// Name:        Settings.cpp
// Purpose:     
// Author:      jrg piringer
// Modified by: 
// Created:     04/05/2006 23:28:28
// RCS-ID:      
// Copyright:   
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "settings.h"

#include <wx/arrimpl.cpp> // this is a magic incantation which must be done!
WX_DEFINE_OBJARRAY(ArrayOfStorageSettings);

IMPLEMENT_DYNAMIC_CLASS(Settings, wxObject);
IMPLEMENT_DYNAMIC_CLASS(SettingStorage, wxObject);

Settings *Settings::settings = NULL;
wxDocManager *Settings::docManager = NULL;

//static Settings setngs;

wxPGProperty *SettingStorage::getProperty()
{
	switch(type)
	{
	case BOOL_STORAGE :
		return wxBoolProperty(description,path,bgf());
	case LONG_STORAGE :
		return wxIntProperty(description,path,lgf());
	case STRING_STORAGE :
		return wxStringProperty(description,path,sgf());
	default :
		return wxStringProperty(_T("ERROR"),path,_T("ERROR"));
	}
}

void SettingStorage::storeValue(wxPropertyGridManager *pgm)
{
	switch(type)
	{
	case BOOL_STORAGE :
		bsf(pgm->GetPropertyValueAsBool(path));
		break;
	case LONG_STORAGE :
		lsf(pgm->GetPropertyValueAsLong(path));
		break;
	case STRING_STORAGE :
		ssf(pgm->GetPropertyValueAsString(path));
		break;
	default :
		wxASSERT_MSG(false,_T("THIS SHOULD NEVER HAPPEN"));
	}
}

/// Settings

Settings::Settings()
{
	config = new wxFileConfig(_T("OSCSurface"),_T("Joerg Piringer"));

	settings = this;
}

Settings::~Settings()
{
	for(size_t i = 0; i < settings->storageSettings.GetCount(); i++)
	{
		SettingStorage *st = settings->storageSettings[i];

		delete st;

		settings->storageSettings[i] = NULL;
	}
	storageSettings.Clear();
}

Settings::Settings(void_func vf)
{
}

void Settings::checkInit()
{
	if(settings == NULL)
	{
		settings = new Settings;
	}
}

SettingStorage *Settings::reg(wxString path, wxString page, wxString cat, wxString desc, storage_type t)
{
	SettingStorage *st = new SettingStorage;

	st->type = t;
	st->path = path;
	st->page = page;
	st->category = cat;
	st->description = desc;

	return st;
}

bool Settings::bool_reg(wxString path, wxString page, wxString cat, wxString desc, bool_getfunc *gf, bool_setfunc *sf, bool std)
{
	SettingStorage *st = reg(path, page, cat, desc, BOOL_STORAGE);

	st->bgf = gf;
	st->bsf = sf;

	checkInit();
	settings->storageSettings.Add(st);

	return std;
}

long Settings::long_reg(wxString path, wxString page, wxString cat, wxString desc, long_getfunc *gf, long_setfunc *sf, long std)
{
	SettingStorage *st = reg(path, page, cat, desc, LONG_STORAGE);

	st->lgf = gf;
	st->lsf = sf;

	checkInit();
	settings->storageSettings.Add(st);

	return std;
}

wxString Settings::string_reg(wxString path, wxString page, wxString cat, wxString desc, string_getfunc *gf, string_setfunc *sf, wxString std)
{
	SettingStorage *st = reg(path, page, cat, desc, STRING_STORAGE);

	st->sgf = gf;
	st->ssf = sf;

	checkInit();
	settings->storageSettings.Add(st);

	return std;
}

size_t findListbookPage(wxListbook *listbook, wxString pagename)
{
	for(size_t i = 0; i < listbook->GetListView()->GetItemCount(); i++)
		if(listbook->GetPageText(i) == pagename)
			return i;
	return -1;
}

void Settings::populateProperties(wxPropertyGridManager *pgm, wxBitmap **bitmap)
{
	//size_t page = -1;
	int bmpnr = 0;

	if(settings->config != NULL)
	{
		for(size_t i = 0; i < settings->storageSettings.GetCount(); i++)
		{
			SettingStorage *st = settings->storageSettings[i];

			if(!st->page.IsEmpty())
			{
				if(pgm->GetPageByName(st->page) != wxNOT_FOUND)
				{
					pgm->SetTargetPage(st->page);
					if(!st->category.IsEmpty())
						if(pgm->IsPropertyCategory(st->category))
							pgm->SetCurrentCategory(st->category);
						else
							pgm->AppendCategory(st->category);

					pgm->Append(st->getProperty());
				}
				else
				{
					if(st->page == _T("General"))
						bmpnr = -1; //SETGENERAL;
					else if(st->page == _T("Audio/MIDI"))
						bmpnr = SETAUDIOMIDI;
					else if(st->page == _T("FP"))
						bmpnr = SETFP;

					if(bmpnr == -1)
						pgm->AddPage(st->page);
					else
						pgm->AddPage(st->page,*(bitmap[bmpnr]));
					if(!st->category.IsEmpty())
						pgm->AppendCategory(st->category);
					pgm->Append(st->getProperty());
				}
			}
		}
	}
}

void Settings::storeProperties(wxPropertyGridManager *pgm)
{
	if(settings->config != NULL)
	{
		for(size_t i = 0; i < settings->storageSettings.GetCount(); i++)
		{
			SettingStorage *st = settings->storageSettings[i];

			if(!st->page.IsEmpty())
			{
				st->storeValue(pgm);
			}
		}
	}
	pgm->ClearModifiedStatus();

	save();
}

void Settings::init(wxDocManager *dM)
{
	setDocManager(dM);

	if(settings->config != NULL)
	{
		for(size_t i = 0; i < settings->storageSettings.GetCount(); i++)
		{
			SettingStorage *st = settings->storageSettings[i];
			long l = 0;
			wxString s;
			bool b = false;

			switch(st->type)
			{
			case BOOL_STORAGE :
				if(settings->config->Read(st->path,&b))
					st->bsf(b);
				break;
			case LONG_STORAGE :
				if(settings->config->Read(st->path,&l))
					st->lsf(l);
				break;
			case STRING_STORAGE :
				if(settings->config->Read(st->path,&s))
					st->ssf(s);
				break;
			default :
				wxASSERT_MSG(false,_T("This should not happen!"));
				break;
			}
		}

		// Load file history
		if(docManager != NULL)
		{
			settings->config->SetPath(wxString(_T("/FileHistory")));
			docManager->FileHistoryLoad(*(settings->config));
		}
	}
}

void Settings::close()
{
	save();

	if(settings->config != NULL)
		delete settings->config;
	settings->config = NULL;

	if(settings != NULL)
		delete settings;
	settings = NULL;
}

void Settings::save()
{
	if(settings->config != NULL)
	{
		for(size_t i = 0; i < settings->storageSettings.GetCount(); i++)
		{
			SettingStorage *st = settings->storageSettings[i];
			long l = 0;
			wxString s;
			bool b = false;

			switch(st->type)
			{
			case BOOL_STORAGE :
				b = st->bgf();
				settings->config->Write(st->path,b);
				break;
			case LONG_STORAGE :
				l = st->lgf();
				settings->config->Write(st->path,l);
				break;
			case STRING_STORAGE :
				s = st->sgf();
				settings->config->Write(st->path,s);
				break;
			default :
				wxASSERT_MSG(false,_T("This should not happen!"));
				break;
			}
		}

		if(docManager != NULL)
		{
			settings->config->SetPath(wxString(_T("/FileHistory")));
			docManager->FileHistorySave(*(settings->config));
		}

		settings->config->Flush();
	}
}
