#ifndef _SETTINGS_H

#define _SETTINGS_H

#include <wx/wx.h>

#include <wx/config.h>
#include <wx/fileconf.h>
#include <wx/propgrid/propgrid.h>

#include <wx/listbook.h>
#include <wx/listctrl.h>

#include <wx/docview.h>

#include <wx/propgrid/manager.h>

typedef void void_func();

typedef bool bool_getfunc();
typedef void bool_setfunc(bool);

typedef long long_getfunc();
typedef void long_setfunc(long);

typedef wxString string_getfunc();
typedef void string_setfunc(wxString);

enum storage_type {BOOL_STORAGE, LONG_STORAGE, STRING_STORAGE};

class SettingStorage : public wxObject
{
	DECLARE_CLASS(SettingStorage);
public:
	storage_type type;
	wxString path;
	wxString page;
	wxString category;
	wxString description;

	union
	{
		bool_getfunc *bgf;
		long_getfunc *lgf;
		string_getfunc *sgf;
	};

	union
	{
		bool_setfunc *bsf;
		long_setfunc *lsf;
		string_setfunc *ssf;
	};

	wxPGProperty *getProperty();
	void storeValue(wxPropertyGridManager *pgm);
};

WX_DECLARE_OBJARRAY(SettingStorage *, ArrayOfStorageSettings);

enum {SETGENERAL = 0, SETAUDIOMIDI, SETFP, __LASTSETBITMAP};

class Settings : public wxObject
{
	DECLARE_DYNAMIC_CLASS(Settings);

	static Settings *settings;
	static wxDocManager *docManager;

	wxFileConfig *config;

	ArrayOfStorageSettings storageSettings;

public:
	Settings();
	Settings(void_func *vf);
	~Settings();

	static void checkInit();
	static bool bool_reg(wxString path, wxString page, wxString cat, wxString desc, bool_getfunc *gf, bool_setfunc *sf, bool std = false);
	static long long_reg(wxString path, wxString page, wxString cat, wxString desc, long_getfunc *gf, long_setfunc *sf, long std = 0);
	static wxString string_reg(wxString path, wxString page, wxString cat, wxString desc, string_getfunc *gf, string_setfunc *sf, wxString std = _T(""));

	static void populateProperties(wxPropertyGridManager *pgm, wxBitmap **bitmap);
	static void storeProperties(wxPropertyGridManager *pgm);

	static void setDocManager(wxDocManager *dM) {docManager = dM;}

	static void init(wxDocManager *dM);
	static void close();
	static void save();

protected:
	static SettingStorage *reg(wxString path, wxString page, wxString cat, wxString desc, storage_type t);
};

#define SETTING_LONG(member)	public: \
								static long member; \
								static long __get_##member() {return member;} \
								static void __set_##member(long _member) {member = _member;}

#define	SET_LONG(cls,member,page,cat,desc)		long cls::member = Settings::long_reg(wxString(_T("/" #cls "/" #member )), wxString(_T(page)), wxString(_T(cat)), wxString(_T(desc)), &cls::__get_##member, &cls::__set_##member);
#define	SET_LONG_I(cls,member,page,cat,desc,init)	long cls::member = Settings::long_reg(wxString(_T("/" #cls "/" #member )), wxString(_T(page)), wxString(_T(cat)), wxString(_T(desc)), &cls::__get_##member, &cls::__set_##member,init);

#define SETTING_BOOL(member)	public: \
								static bool member; \
								static bool __get_##member() {return member;} \
								static void __set_##member(bool _member) {member = _member;}

#define	SET_BOOL(cls,member,page,cat,desc)		bool cls::member = Settings::bool_reg(wxString(_T("/" #cls "/" #member )), wxString(_T(page)), wxString(_T(cat)), wxString(_T(desc)), &cls::__get_##member, &cls::__set_##member);
#define	SET_BOOL_I(cls,member,page,cat,desc,init)	bool cls::member = Settings::bool_reg(wxString(_T("/" #cls "/" #member )), wxString(_T(page)), wxString(_T(cat)), wxString(_T(desc)), &cls::__get_##member, &cls::__set_##member,init);

#define SETTING_STRING(member)	public: \
								static wxString member; \
								static wxString __get_##member() {return member;} \
								static void __set_##member(wxString _member) {member = _member;}

#define	SET_STRING(cls,member,page,cat,desc)			wxString cls::member = Settings::string_reg(wxString(_T("/" #cls "/" #member )), wxString(_T(page)), wxString(_T(cat)), wxString(_T(desc)),  &cls::__get_##member, &cls::__set_##member);
#define	SET_STRING_I(cls,member,page,cat,desc,init)	    wxString cls::member = Settings::string_reg(wxString(_T("/" #cls "/" #member )), wxString(_T(page)), wxString(_T(cat)), wxString(_T(desc)), &cls::__get_##member, &cls::__set_##member,_T(init));

#define SETTING_GET(cls,member)			cls::__get_##member()
#define SETTING_SET(cls,member,st)		cls::__set_##member(st)

extern size_t findListbookPage(wxListbook *listbook, wxString pagename);

#endif

